-----------------------------------------------------------------------------------------------
-- Client Lua Script for DyeTracker
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Apollo"
require "GameLib"
require "Challenges"
require "ChallengesLib"
require "Window"
require "Item"

 
-----------------------------------------------------------------------------------------------
-- DyeTracker Module Definition
-----------------------------------------------------------------------------------------------
local DyeTracker = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999


local tItemQuality =
{
	{ ["nIndex"] = Item.CodeEnumItemQuality.Inferior,		["sName"] = Apollo.GetString("CRB_Inferior"),		["sColor"] = "ItemQuality_Inferior" },
	{ ["nIndex"] = Item.CodeEnumItemQuality.Average,		["sName"] = Apollo.GetString("CRB_Average"),		["sColor"] = "ItemQuality_Average" },
	{ ["nIndex"] = Item.CodeEnumItemQuality.Good,			["sName"] = Apollo.GetString("CRB_Good"),			["sColor"] = "ItemQuality_Good" },
	{ ["nIndex"] = Item.CodeEnumItemQuality.Excellent,		["sName"] = Apollo.GetString("CRB_Excellent"),		["sColor"] = "ItemQuality_Excellent" },
	{ ["nIndex"] = Item.CodeEnumItemQuality.Superb,			["sName"] = Apollo.GetString("CRB_Superb"),			["sColor"] = "ItemQuality_Superb" },
	{ ["nIndex"] = Item.CodeEnumItemQuality.Legendary,		["sName"] = Apollo.GetString("CRB_Legendary"),		["sColor"] = "ItemQuality_Legendary" },
	{ ["nIndex"] = Item.CodeEnumItemQuality.Artifact,		["sName"] = Apollo.GetString("CRB_Artifact"),		["sColor"] = "ItemQuality_Artifact" }
}

local tMasterDyesDev =
{
	{ ["sName"] = "Spirovine",			["nIndex"] = 1,		["nId"] = 2,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Laser Beam",			["nIndex"] = 2,		["nId"] = 3,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Dominion Red",		["nIndex"] = 3,		["nId"] = 4,	["nQuality"] = 5,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Bloodfire Red",		["nIndex"] = 5, 	["nId"] = 6,	["nQuality"] = 4,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Faded Pink",			["nIndex"] = 6,		["nId"] = 7,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Coralscale",			["nIndex"] = 7,		["nId"] = 8,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 3, ["nIndex"] = 1 } } },
	{ ["sName"] = "Redshift",			["nIndex"] = 8,		["nId"] = 9,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 2, ["nIndex"] = 2 }, { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Near-Infrared",		["nIndex"] = 9,		["nId"] = 10,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Redmoon",			["nIndex"] = 10,	["nId"] = 11,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },	
	{ ["sName"] = "Xenocite",			["nIndex"] = 11,	["nId"] = 12,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },	
	{ ["sName"] = "Hyperspace",			["nIndex"] = 14,	["nId"] = 15,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },		
	{ ["sName"] = "Misty Morning",		["nIndex"] = 16,	["nId"] = 17,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },	
	{ ["sName"] = "Mythistle",			["nIndex"] = 17,	["nId"] = 18,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 3, ["nIndex"] = 1 } } },
	{ ["sName"] = "Falkrin Fuchsia",	["nIndex"] = 18,	["nId"] = 19,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Orbital Orchid",		["nIndex"] = 19,	["nId"] = 20,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Blood Orchid",		["nIndex"] = 20,	["nId"] = 21,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Penumbra",			["nIndex"] = 21,	["nId"] = 22,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 3, ["nIndex"] = 1 } } },
	{ ["sName"] = "Parsec Purple",		["nIndex"] = 22,	["nId"] = 23,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 2, ["nIndex"] = 1 } } },
	{ ["sName"] = "Zero Gravity",		["nIndex"] = 24,	["nId"] = 25,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Umbra",				["nIndex"] = 25,	["nId"] = 26,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Hazy Purple",		["nIndex"] = 26,	["nId"] = 27,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Vacuum Violet",		["nIndex"] = 27,	["nId"] = 28,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },	
	{ ["sName"] = "Artificial Gravity",	["nIndex"] = 28,	["nId"] = 29,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Darkspur Purple",	["nIndex"] = 30,	["nId"] = 31,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Celestial Light",	["nIndex"] = 31,	["nId"] = 32,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Psionic Purple",		["nIndex"] = 32,	["nId"] = 33,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },	
	{ ["sName"] = "Purple Strain",		["nIndex"] = 33,	["nId"] = 34,	["nQuality"] = 4,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 3 } } },	
	{ ["sName"] = "Dark Sky",			["nIndex"] = 34,	["nId"] = 35,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Dismal Drizzle",		["nIndex"] = 37,	["nId"] = 38,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 3, ["nIndex"] = 1 } } },
	{ ["sName"] = "Extragalactic",		["nIndex"] = 35,	["nId"] = 36,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Cosmic Ray",			["nIndex"] = 36,	["nId"] = 37,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Starflight",			["nIndex"] = 38,	["nId"] = 39,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Reactor Smoke",		["nIndex"] = 39,	["nId"] = 40,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Deep Space",			["nIndex"] = 40,	["nId"] = 41,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Skeech Blue",		["nIndex"] = 41,	["nId"] = 42,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },	
	{ ["sName"] = "Comet Blue",			["nIndex"] = 42,	["nId"] = 43,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Ultramarine",		["nIndex"] = 45,	["nId"] = 46,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Loftite Blue",		["nIndex"] = 46,	["nId"] = 47,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 3, ["nIndex"] = 1 } } },
	{ ["sName"] = "Absolute Zero",		["nIndex"] = 47,	["nId"] = 48,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Blueshift",			["nIndex"] = 48,	["nId"] = 49,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 2, ["nIndex"] = 2 }, { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Torine Blue",		["nIndex"] = 51,	["nId"] = 52,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Exile Blue",			["nIndex"] = 52,	["nId"] = 53,	["nQuality"] = 4,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Cold Fusion",		["nIndex"] = 54,	["nId"] = 55,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 2, ["nIndex"] = 6 }, { ["nType"] = 2, ["nIndex"] = 9 } } },
	{ ["sName"] = "Maris Blue",			["nIndex"] = 55,	["nId"] = 56,	["nQuality"] = 3,	["sSourceName"] = "Cold Cobalt",	["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Galactium",			["nIndex"] = 56,	["nId"] = 57,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Vitalus Blue",		["nIndex"] = 57,	["nId"] = 58,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },	
	{ ["sName"] = "Ikthian Blue",		["nIndex"] = 58,	["nId"] = 59,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Hydrogem",			["nIndex"] = 59,	["nId"] = 60,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Undersea Cavern",	["nIndex"] = 60,	["nId"] = 61,	["nQuality"] = 2,	["sSourceName"] = "Undersea",		["Sources"] = {} },
	{ ["sName"] = "Protostar Green",	["nIndex"] = 61,	["nId"] = 62,	["nQuality"] = 4,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Arboria Green",		["nIndex"] = 62,	["nId"] = 63,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Bladeleaf",			["nIndex"] = 64,	["nId"] = 65,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 2, ["nIndex"] = 2 }, { ["nType"] = 2, ["nIndex"] = 5 } } },
	{ ["sName"] = "Undergrowth",		["nIndex"] = 65,	["nId"] = 66,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Zephyrite",			["nIndex"] = 66,	["nId"] = 67,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 3, ["nIndex"] = 1 } } },
	{ ["sName"] = "Meadowspring",		["nIndex"] = 67,	["nId"] = 68,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Terraform",			["nIndex"] = 68,	["nId"] = 69,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Bushwhacked",		["nIndex"] = 69,	["nId"] = 70,	["nQuality"] = 2,	["sSourceName"] = "Stumpkin Green",	["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Flux",				["nIndex"] = 71,	["nId"] = 72,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Gamma Ray",			["nIndex"] = 72,	["nId"] = 73,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Augmented Green",	["nIndex"] = 74,	["nId"] = 75,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Novacite",			["nIndex"] = 75,	["nId"] = 76,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Aurora",				["nIndex"] = 76,	["nId"] = 77,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Mountain Green",		["nIndex"] = 77,	["nId"] = 78,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },	
	{ ["sName"] = "Osun Green",			["nIndex"] = 78,	["nId"] = 79,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Brushy Glen",		["nIndex"] = 79,	["nId"] = 80,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },	
	{ ["sName"] = "Flightsuit Green",	["nIndex"] = 80,	["nId"] = 81,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Shiny Yellow",		["nIndex"] = 81,	["nId"] = 82,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Shockwave",			["nIndex"] = 82,	["nId"] = 83,	["nQuality"] = 4,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 3 } } },
	{ ["sName"] = "Ionic Yellow",		["nIndex"] = 83,	["nId"] = 84,	["nQuality"] = 5,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Osun Ale",			["nIndex"] = 84,	["nId"] = 85,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Astro Olive",		["nIndex"] = 85,	["nId"] = 86,	["nQuality"] = 2,	["sSourceName"] = "Astrodrab",		["Sources"] = {} },
	{ ["sName"] = "Sunrise",			["nIndex"] = 86,	["nId"] = 87,	["nQuality"] = 2,	["sSourceName"] = "Dusky Sunrise",	["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Primal Air",			["nIndex"] = 87,	["nId"] = 88,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Pell Yellow",		["nIndex"] = 88,	["nId"] = 89,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Engine Oil",			["nIndex"] = 89,	["nId"] = 90,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },	
	{ ["sName"] = "Commando",			["nIndex"] = 90,	["nId"] = 91,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Glittering Gold",	["nIndex"] = 91,	["nId"] = 92,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Primal Earth",		["nIndex"] = 92,	["nId"] = 93,	["nQuality"] = 4,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Honeywheat",			["nIndex"] = 93,	["nId"] = 94,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 2, ["nIndex"] = 1 } } },
	{ ["sName"] = "Bingberry",			["nIndex"] = 94,	["nId"] = 95,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },	
	{ ["sName"] = "Dark Gold",			["nIndex"] = 95,	["nId"] = 96,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Exanite",			["nIndex"] = 96,	["nId"] = 97,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Eldan Gold",			["nIndex"] = 97,	["nId"] = 98,	["nQuality"] = 2,	["sSourceName"] = "Battlebot Beige",	["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Sandstorm",			["nIndex"] = 98,	["nId"] = 99,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Amberwood",			["nIndex"] = 99,	["nId"] = 100,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Ironbark",			["nIndex"] = 100,	["nId"] = 101,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },	
	{ ["sName"] = "Afterburn",			["nIndex"] = 101,	["nId"] = 102,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 3, ["nIndex"] = 1 } } },
	{ ["sName"] = "Starburst",			["nIndex"] = 102,	["nId"] = 103,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Fireball",			["nIndex"] = 103,	["nId"] = 104,	["nQuality"] = 4,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Grimgourd",			["nIndex"] = 104,	["nId"] = 105,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Generator Ginger",	["nIndex"] = 105,	["nId"] = 106,	["nQuality"] = 3,	["sSourceName"] = "Megadroid Umber",	["Sources"] = { { ["nType"] = 2, ["nIndex"] = 1 }, { ["nType"] = 2, ["nIndex"] = 5 }, { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Cardboard Box",		["nIndex"] = 106,	["nId"] = 107,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Camelope",			["nIndex"] = 107,	["nId"] = 108,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Corona",				["nIndex"] = 108,	["nId"] = 109,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Overbronze",			["nIndex"] = 109,	["nId"] = 110,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Tough Mudder",		["nIndex"] = 110,	["nId"] = 111,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },	
	{ ["sName"] = "Orange Spark",		["nIndex"] = 111,	["nId"] = 112,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Solar Flare",		["nIndex"] = 112,	["nId"] = 113,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Primal Flame",		["nIndex"] = 113,	["nId"] = 114,	["nQuality"] = 5,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 4 } } },
	{ ["sName"] = "Freebot Rust",		["nIndex"] = 114,	["nId"] = 115,	["nQuality"] = 3,	["sSourceName"] = "Rusty Freebot",	["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Ochre Jam",			["nIndex"] = 115,	["nId"] = 116,	["nQuality"] = 3,	["sSourceName"] = "Blood Ochre",	["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Hotdrop Orange",		["nIndex"] = 116,	["nId"] = 117,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },		
	{ ["sName"] = "Crater Stone",		["nIndex"] = 117,	["nId"] = 118,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },		
	{ ["sName"] = "Terminator Line",	["nIndex"] = 118,	["nId"] = 119,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Spice Freighter",	["nIndex"] = 120,	["nId"] = 121,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Nova Grey",			["nIndex"] = 121,	["nId"] = 122,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Platinum Grey",		["nIndex"] = 122,	["nId"] = 123,	["nQuality"] = 2,	["sSourceName"] = "Platinum Gray",	["Sources"] = {} },
	{ ["sName"] = "Event Horizon",		["nIndex"] = 124,	["nId"] = 125,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Dark Matter",		["nIndex"] = 125,	["nId"] = 126,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Scud Grey",			["nIndex"] = 126,	["nId"] = 127,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Granok Stone",		["nIndex"] = 127,	["nId"] = 128,	["nQuality"] = 2,	["sSourceName"] = "Gnox Stone",		["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Stony Iron",			["nIndex"] = 128,	["nId"] = 129,	["nQuality"] = 2,	["sSourceName"] = "Gnox Iron",		["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Solar Eclipse",		["nIndex"] = 129,	["nId"] = 130,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 2, ["nIndex"] = 1 }, { ["nType"] = 2, ["nIndex"] = 2 } } },
	{ ["sName"] = "Dark Energy",		["nIndex"] = 130,	["nId"] = 131,	["nQuality"] = 4,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Lunar Light",		["nIndex"] = 131,	["nId"] = 132,	["nQuality"] = 2,	["sSourceName"] = "Halon Gray",		["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Mechari Metal",		["nIndex"] = 132,	["nId"] = 133,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Anti-Matter",		["nIndex"] = 133,	["nId"] = 134,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Lunar Eclipse",		["nIndex"] = 134,	["nId"] = 135,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Eldan Timberwood",	["nIndex"] = 136,	["nId"] = 137,	["nQuality"] = 2,	["sSourceName"] = "Timberwood",		["Sources"] = {} },
	{ ["sName"] = "Rowsbrowner",		["nIndex"] = 137,	["nId"] = 138,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 3, ["nIndex"] = 1 } } },
	{ ["sName"] = "Space Leather",		["nIndex"] = 138,	["nId"] = 139,	["nQuality"] = 3,	["sSourceName"] = "Roanhide Russet",	["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Asteroid",			["nIndex"] = 139,	["nId"] = 140,	["nQuality"] = 2,	["sSourceName"] = "Hard Asteroid",	["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },		
	{ ["sName"] = "Heartwood",			["nIndex"] = 140,	["nId"] = 141,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 2, ["nIndex"] = 1 }, { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Pink Fury",			["nIndex"] = 141,	["nId"] = 142,	["nQuality"] = 4,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Marooned",			["nIndex"] = 145,	["nId"] = 146,	["nQuality"] = 4,	["sSourceName"] = "Infradread",		["Sources"] = {} },
	{ ["sName"] = "Saltwater Khaki",	["nIndex"] = 146,	["nId"] = 147,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Bingberry Leaf",		["nIndex"] = 147,	["nId"] = 148,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Tarnished Contact",	["nIndex"] = 148,	["nId"] = 149,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 3, ["nIndex"] = 1 } } },
	{ ["sName"] = "Jabbithide Tan",		["nIndex"] = 149,	["nId"] = 150,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = {} },
	{ ["sName"] = "Hidden Hollow",		["nIndex"] = 150,	["nId"] = 151,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = {} },		
	{ ["sName"] = "Lopp Fur",			["nIndex"] = 151,	["nId"] = 152,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Totality Taupe",		["nIndex"] = 152,	["nId"] = 153,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 3, ["nIndex"] = 1 } } },
	{ ["sName"] = "Ancient Rust",		["nIndex"] = 153,	["nId"] = 154,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 2 } } },
	{ ["sName"] = "Oxidized",			["nIndex"] = 154,	["nId"] = 155,	["nQuality"] = 2,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 4, ["nIndex"] = 1 } } },
	{ ["sName"] = "Deluxe Eldan Green",	["nIndex"] = 161,	["nId"] = 344,	["nQuality"] = 5,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 5, ["nIndex"] = 1 } } },
	{ ["sName"] = "Corruption Purple",	["nIndex"] = 163,	["nId"] = 346,	["nQuality"] = 3,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 3, ["nIndex"] = 2 } } },
	{ ["sName"] = "Deluxe Eldan Gold",	["nIndex"] = 168,	["nId"] = 351,	["nQuality"] = 5,	["sSourceName"] = "",				["Sources"] = { { ["nType"] = 5, ["nIndex"] = 1 } } }
}

local tSourceTypes = 
{
	{	["nIndex"] = 1, ["sName"] = "Challenge" },
	{	["nIndex"] = 2, ["sName"] = "Collection" },
	{	["nIndex"] = 3, ["sName"] = "Vendor" },
	{	["nIndex"] = 4, ["sName"] = "Box" },	
	{	["nIndex"] = 5, ["sName"] = "Misc" },	
}

local tChallenges = 
{
	{ ["nId"] = 500,	["nType"] = 2,	["sName"] = "Murgh Along",	["sDescription"] = "Reach the cliff overlooking Stormwind Fortress",	["sLocation"] = "Stormwing Fortress :Stormwind Fortress",	["tRewards"] = {} }
}

local tCollections = 
{
	{ ["sName"] = "Corruption",		["nIndex"] = 1,	["nQuality"] = 3,	["tDyes"] = { { ["nIndex"] =  22 }, { ["nIndex"] =  93 }, { ["nIndex"] = 105 }, { ["nIndex"] = 129 }, { ["nIndex"] = 140 } }, ["tSources"] = {} },
	{ ["sName"] = "Doon",			["nIndex"] = 2,	["nQuality"] = 3,	["tDyes"] = { { ["nIndex"] =  64 }, { ["nIndex"] =  48 }, { ["nIndex"] = 108 }, { ["nIndex"] =   8 }, { ["nIndex"] = 129 } }, ["tSources"] = {} },
	{ ["sName"] = "Fringerunner",	["nIndex"] = 3,	["nQuality"] = 3, 	["tDyes"] = { { ["nIndex"] =  48 }, { ["nIndex"] = 118 }, { ["nIndex"] =  65 }, { ["nIndex"] =  28 }, { ["nIndex"] = 124 } }, ["tSources"] = {} },
	{ ["sName"] = "Highborn",		["nIndex"] = 4,	["nQuality"] = 3, 	["tDyes"] = { { ["nIndex"] =  28 }, { ["nIndex"] =  14 }, { ["nIndex"] = 149 }, { ["nIndex"] = 102 }, { ["nIndex"] =  65 } }, ["tSources"] = {} },
	{ ["sName"] = "Ikthia",			["nIndex"] = 5,	["nQuality"] = 3, 	["tDyes"] = { { ["nIndex"] =  64 }, { ["nIndex"] =  42 }, { ["nIndex"] = 105 }, { ["nIndex"] = 121 }, { ["nIndex"] =  22 } }, ["tSources"] = {} },
	{ ["sName"] = "Kel Voreth",		["nIndex"] = 6,	["nQuality"] = 3, 	["tDyes"] = { { ["nIndex"] =  54 }, { ["nIndex"] =  35 }, { ["nIndex"] =   9 }, { ["nIndex"] =  84 }, { ["nIndex"] = 102 } }, ["tSources"] = {} },
	{ ["sName"] = "Lowborn",		["nIndex"] = 7,	["nQuality"] = 3, 	["tDyes"] = { { ["nIndex"] = 118 }, { ["nIndex"] =  84 }, { ["nIndex"] = 115 }, { ["nIndex"] =  35 }, { ["nIndex"] = 134 } }, ["tSources"] = {} },
	{ ["sName"] = "Stonebreaker",	["nIndex"] = 8,	["nQuality"] = 3, 	["tDyes"] = { { ["nIndex"] = 124 }, { ["nIndex"] =  71 }, { ["nIndex"] = 140 }, { ["nIndex"] =  93 }, { ["nIndex"] = 149 } }, ["tSources"] = {} },
	{ ["sName"] = "Stormtalon",		["nIndex"] = 9,	["nQuality"] = 3, 	["tDyes"] = { { ["nIndex"] =  54 }, { ["nIndex"] = 115 }, { ["nIndex"] = 121 }, { ["nIndex"] = 108 }, { ["nIndex"] =   8 } }, ["tSources"] = {} }
}

local tVendors =
{
	{ ["nIndex"] = 1, ["sName"] = "Dye Vendor (Thayd/Illium)" },
	{ ["nIndex"] = 2, ["sName"] = "NEED NAME (Blighthaven)" }	
}

local tBoxes =
{
	{ ["nIndex"] = 1, ["sName"] = "Trunk of Tarnished Tints",			["nQuality"] = 3 },
	{ ["nIndex"] = 2, ["sName"] = "Case of Coordinated Colors",			["nQuality"] = 4 },
	{ ["nIndex"] = 3, ["sName"] = "Duffel Bag of Distinguished Dyes",	["nQuality"] = 4 },
	{ ["nIndex"] = 4, ["sName"] = "Pouch of Pristine Pigments",			["nQuality"] = 5 },
	{ ["nIndex"] = 5, ["sName"] = "Boom-Box",							["nQuality"] = 5 },
}


local tMisc =
{
	{ ["nIndex"] = 1, ["sName"] = "Deluxe Edition" }
}

local tDyeCollections =
{
	{ ["sName"] = "Corruption", ["Dye1"] = "Heartwood", ["Dye2"] = "Honeywheat", ["Dye3"] = "Generator Ginger", ["Dye4"] = "Parsec Purple", ["Dye5"] = "Solar Eclipse", ["Quality"] = "Good", ["Source"] = "FABkit Challenge: Dump Diving\nFABkit Challenge: Medic!" },
	{ ["sName"] = "Doon", ["Dye1"] = "Bladeleaf", ["Dye2"] = "Blueshift", ["Dye3"] = "Corona", ["Dye4"] = "Redshift", ["Dye5"] = "Solar Eclipse", ["Quality"] = "Good", ["Source"] = "Mossy Reward Bag (Abandoned Test Lab)" },
	{ ["sName"] = "Fringerunner", ["Dye1"] = "Blueshift", ["Dye2"] = "Terminator Line", ["Dye3"] = "Undergrowth", ["Dye4"] = "Artificial Gravity", ["Dye5"] = "Event Horizon", ["Quality"] = "Good", ["Source"] = "VENDOR: Renown Thayd/Illium" },
	{ ["sName"] = "Highborn", ["Dye1"] = "Artificial Gravity", ["Dye2"] = "Hyperspace", ["Dye3"] = "Jabbithide Tan", ["Dye4"] = "Starburst", ["Dye5"] = "Undergrowth", ["Quality"] = "Good", ["Source"] = "FABkit Challenge: Cracking the Code\nFABkit Challenge: Eggbeater\nFABkit Challenge: Shiny Vind Chase\nFABkit Challenge: Evasive Maneuvers\nFABkit Challenge: Top of the World" },
	{ ["sName"] = "Ikthia", ["Dye1"] = "Bladeleaf", ["Dye2"] = "Comet Blue", ["Dye3"] = "Generator Ginger", ["Dye4"] = "Nova Grey", ["Dye5"] = "Parsec Purple", ["Quality"] = "Good", ["Source"] = "FABkit Challenge: That's the Spirit\nFABkit Challenge: Ice Anomalies\nFABkit Challenge: Grounding the Storm\nFABkit Challenge: Rider on the Storm" },
	{ ["sName"] = "Kel Voreth", ["Dye1"] = "Cold Fusion", ["Dye2"] = "Extragalactic", ["Dye3"] = "Near-Infrared", ["Dye4"] = "Osun Ale", ["Dye5"] = "Starburst", ["Quality"] = "Good", ["Source"] = "Renown Vendor (Thayd/Illium)\nFrayed Reward Bag (Mayday Expedition)" },
	{ ["sName"] = "Lowborn", ["Dye1"] = "Terminator Line", ["Dye2"] = "Osun Ale", ["Dye3"] = "Ochre Jam", ["Dye4"] = "Extragalactic", ["Dye5"] = "Lunar Eclipse", ["Quality"] = "Good", ["Source"] = "Sinister Reward Bag (Creepy Cave Expedition)" },
	{ ["sName"] = "Stonebreaker", ["Dye1"] = "Event Horizon", ["Dye2"] = "Flux", ["Dye3"] = "Heartwood", ["Dye4"] = "Honeywheat", ["Dye5"] = "Jabbithide Tan", ["Quality"] = "Good", ["Source"] = "FABkit Challenge: Intercept the Ikthians\nFABkit Challenge: Steamglider Flurry\nRagged Reward Bag (Blasted Landscape)" },
	{ ["sName"] = "Stormtalon", ["Dye1"] = "Cold Fusion", ["Dye2"] = "Ochre Jam", ["Dye3"] = "Nova Grey", ["Dye4"] = "Corona", ["Dye5"] = "Redshift", ["Quality"] = "Good", ["Source"] = "Singed Reward Bag (Kel Voreth Expedition)" }
}
	--{ ["sName"] = "", ["Dye1"] = "", ["Dye2"] = "", ["Dye3"] = "", ["Dye4"] = "", ["Dye5"] = "", ["Quality"] = "", ["Source"] = "" }
 
local bFirstShow = true	

local tCurrentDyes
	
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function DyeTracker:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function DyeTracker:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = "DyeTracker"
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- DyeTracker OnLoad
-----------------------------------------------------------------------------------------------
function DyeTracker:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("DyeTracker.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- DyeTracker OnDocLoaded
-----------------------------------------------------------------------------------------------
function DyeTracker:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "DyeTrackerForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("DyeTrackerInterfaceList", "OnConfigure", self)
		
		Apollo.RegisterSlashCommand("dt", "OnDyeTrackerOn", self)
		Apollo.RegisterSlashCommand("dyetracker", "OnDyeTrackerOn", self)


		-- Do additional Addon initialization here
		Apollo.RegisterEventHandler("DyeLearned", "OnDyeLearned", self)
		
		--self:WriteDebug()
		self.wndMain:FindChild("MainNavDyes"):SetCheck(true)
		self.wndMain:FindChild("ToggleIndexBtn"):SetCheck(true)
		self.wndMain:FindChild("ToggleKnownBtn"):SetCheck(true)
		self.wndMain:FindChild("ToggleUnknownBtn"):SetCheck(true)
		self.wndMain:FindChild("ToggleBoxBtn"):SetCheck(true)
		self.wndMain:FindChild("ToggleCollectionBtn"):SetCheck(true)
		self.wndMain:FindChild("ToggleVendorBtn"):SetCheck(true)
		self.wndMain:FindChild("ToggleMiscBtn"):SetCheck(true)
		self.wndMain:FindChild("ToggleMissingBtn"):SetCheck(true)
		self.wndMain:FindChild("ToggleAverageBtn"):SetCheck(true)
		self.wndMain:FindChild("ToggleGoodBtn"):SetCheck(true)
		self.wndMain:FindChild("ToggleExcellentBtn"):SetCheck(true)
		self.wndMain:FindChild("ToggleSuperbBtn"):SetCheck(true)
				
		self:CheckForMissingDyes()
		self:UpdateKnownFlag()
		self:SetVisibleFlag()

	end
end

-----------------------------------------------------------------------------------------------
-- DyeTracker Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/dt"
function DyeTracker:OnDyeTrackerOn()
	if bFirstShow == true then	
		self:FillDyes2()
		bFirstShow = false	
	end

	--START
		local tAllChallenges = ChallengesLib.GetActiveChallengeList()
		
		local sChallenges = ""
		for idx, clgCurrent in pairs(tAllChallenges) do
			sChallenges = sChallenges .. "Id: " .. clgCurrent:GetId() .. "\n"
			sChallenges = sChallenges .. "Type: " .. clgCurrent:GetType() .. "\n"
			sChallenges = sChallenges .. "Name: " .. clgCurrent:GetName() .. "\n"
			sChallenges = sChallenges .. "Description: " .. clgCurrent:GetDescription() .. "\n"
			
			local tZoneRestrictionInfo = clgCurrent:GetZoneRestrictionInfo()
			sLocation = ""
			if tZoneRestrictionInfo.strSubZoneName and tZoneRestrictionInfo.strSubZoneName ~= "" then
				sLocation = tZoneRestrictionInfo.strSubZoneName .. " : " .. tZoneRestrictionInfo.strLocationName
			end

			sChallenges = sChallenges .. "Location: " .. sLocation .. "\n"

			ChallengesLib.GenerateRewardList(clgCurrent:GetId())	
			local tRewardList = ChallengesLib.GetRewardList(clgCurrent:GetId())
			--local tRewardList = ChallengesLib.GetRewardList(self.idChallenge)
			
			for i, tReward in ipairs(tRewardList) do
				if tRewards.itemReward then
					sChallenges = sChallenges .. "Reward Item: " .. tRewards.itemReward .. "\n"
				elseif tRewards.monReward then
					sChallenges = sChallenges .. "Reward Money: " .. tRewards.monReward .. "\n"
					wndChallengeItem:FindChild("LootIcon"):SetMoneyInfo(tRewards.monReward, tRewards.nAmount)
				elseif tRewards.splReward then
					sChallenges = sChallenges .. "Reward Spell: " .. tRewards.splReward .. "\n"
				elseif tRewards.nXp then
					sChallenges = sChallenges .. "Reward XP: " .. tRewards.nXp .. "\n"
				elseif tRewards.nRepId then
					sChallenges = sChallenges .. "Reward Reputation: " .. tRewards.nRepId .. "\n"
				end
						

			
			end
			
			break		
			
		end
		self.wndMain:FindChild("CWindow"):SetText(sChallenges)	
	--END
	
	
	self.wndMain:Invoke() -- show the window
end


-----------------------------------------------------------------------------------------------
-- DyeTrackerForm Functions
-----------------------------------------------------------------------------------------------

-- when the Cancel button is clicked
function DyeTracker:OnCancel()
	self.wndMain:Close() -- hide the window
end

-- Pandemic
function DyeTracker:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "DyeTracker", {"DyeTrackerInterfaceList", "", ""})
	--self:UpdateInterfaceMenuAlerts()
end

function DyeTracker:OnConfigure()
	self:OnDyeTrackerOn()
end


function DyeTracker:OnDyeLearned()
	self:CheckForMissingDyes()
	self:UpdateKnownFlag()
	
	if self.wndMain:FindChild("ToggleIndexBtn"):IsChecked() then
		table.sort(tMasterDyesDev, function (a,b) return a.nIndex < b.nIndex end)
	else
		table.sort(tMasterDyesDev, function (a,b) return a.sName < b.sName end)	
	end
	
	if self.wndMain:FindChild("MainNavDyes"):IsChecked() then
		self:FillDyes2()
	else
		self:FillCollections()
	end
end


function DyeTracker:FillDyes2()
	local nTotal = table.getn(tMasterDyesDev)
	local nCurrent = table.getn(GameLib.GetKnownDyes())

	self.wndMain:FindChild("ProgressText"):SetText(String_GetWeaselString(Apollo.GetString("Lore_TotalProgress"), nCurrent, nTotal))	
	self.wndMain:FindChild("ProgressProgBar"):SetMax(nTotal)
	self.wndMain:FindChild("ProgressProgBar"):SetProgress(nCurrent)
	self.wndMain:FindChild("ProgressProgBar"):EnableGlow(nCurrent == nTotal)
		
	local wndMainScroll = self.wndMain:FindChild("MainScroll")	
	wndMainScroll:DestroyChildren()
	
	for i, tDyeInfo in ipairs(tMasterDyesDev) do
		if tDyeInfo.Visible == true then
			local wndNewDyeItem = Apollo.LoadForm(self.xmlDoc, "DyeItem", wndMainScroll, self)
			local strName = ""
			local sSource = ""

			if tDyeInfo.sName and tDyeInfo.sName:len() > 0 then
				strName = tDyeInfo.sName
			else
				strName = String_GetWeaselString(Apollo.GetString("CRB_CurlyBrackets"), tDyeInfo.nIndex)
			end
		
			local strSprite = "CRB_DyeRampSprites:sprDyeRamp_" .. tDyeInfo.nIndex
			wndNewDyeItem:FindChild("Icon"):SetSprite(strSprite)
			wndNewDyeItem:FindChild("ItemText"):SetText(strName)
			wndNewDyeItem:FindChild("ItemQuality"):SetText(tItemQuality[tDyeInfo.nQuality].sName)
			wndNewDyeItem:FindChild("ItemQuality"):SetTextColor(tItemQuality[tDyeInfo.nQuality].sColor)

			
			if tDyeInfo.sSourceName:len() > 0 then
				sSource = "Source Dye: " .. tDyeInfo.sSourceName .. "\n"
			end
			
			for j, tSource in ipairs(tDyeInfo.Sources) do
				if tSource.nType == 1 then
					sSource = sSource .. tChallenges[tSource.nIndex].sName .. "\n"
				end
				if tSource.nType == 2 then
					sSource = sSource .. tCollections[tSource.nIndex].sName .. " Collection\n"
				end
				if tSource.nType == 3 then
					sSource = sSource .. tVendors[tSource.nIndex].sName .. "\n"
				end
				if tSource.nType == 4 then
					sSource = sSource .. tBoxes[tSource.nIndex].sName .. "\n"
				end
				if tSource.nType == 5 then
					sSource = sSource .. tMisc[tSource.nIndex].sName .. "\n"
				end
			end

			wndNewDyeItem:FindChild("ItemTextSource"):SetText(sSource)
															
			if tDyeInfo.Known == false then
				wndNewDyeItem:FindChild("ItemText"):SetTextColor("darkgray")
			end
		end
	end

	wndMainScroll:ArrangeChildrenTiles()
end


function DyeTracker:CheckForMissingDyes()
	local tKnownDyes = GameLib.GetKnownDyes()

	for j, tKnownDye in ipairs(tKnownDyes) do
		local sDyeName = ""
		local bFound = false

		if tKnownDye.strName and tKnownDye.strName:len() > 0 then
			sDyeName = tKnownDye.strName
		else
			sDyeName = String_GetWeaselString(Apollo.GetString("CRB_CurlyBrackets"), "", tKnownDye.nIndex)
		end

		for i, tMasterDye in ipairs(tMasterDyesDev) do		
			if sDyeName == tMasterDye.sName then
				bFound = true
				break
			end
		end	
		
		if bFound == false then
			table.insert(tMasterDyesDev, { ["sName"] = "MISSING: " .. sDyeName .. ", " .. tKnownDye.nRampIndex .. ", " .. tKnownDye.nId, ["nIndex"] = tKnownDye.nRampIndex, ["nId"] = tKnownDye.nId, ["nQuality"] = 1, ["sSourceName"] = "", ["Sources"] = {} })
		end				
	end
end


function DyeTracker:UpdateKnownFlag()
	local tKnownDyes = GameLib.GetKnownDyes()
	
	for i, tMasterDye in ipairs(tMasterDyesDev) do
		local sDyeName = ""
		local bFound = false
		
		for j, tKnownDye in ipairs(tKnownDyes) do
			if tKnownDye.strName and tKnownDye.strName:len() > 0 then
				sDyeName = tKnownDye.strName
			else
				sDyeName = String_GetWeaselString(Apollo.GetString("CRB_CurlyBrackets"), "", tKnownDye.nIndex)
			end
			
			if sDyeName == tMasterDye.sName then
				bFound = true
				break
			end
		end
		
		tMasterDye.Known = bFound
	end
end

function DyeTracker:FillCollections()
	local wndMainScroll = self.wndMain:FindChild("MainScroll")	
	wndMainScroll:DestroyChildren()

	local tKnownDyes = GameLib.GetKnownDyes()
	
	for i, tCollection in ipairs(tDyeCollections) do
		local wndNewDyeItem = Apollo.LoadForm(self.xmlDoc, "CollectionItem", wndMainScroll, self)
		
		wndNewDyeItem:FindChild("ItemText"):SetText(tCollection.sName)
		
		for j = 1, 5 do
			wndNewDyeItem:FindChild("Dye" .. j):FindChild("ItemText"):SetText(tCollection["Dye" .. j])
			
			local sDyeName = ""
			local bFound = false
			for k, tKnownDye in ipairs(tKnownDyes) do
				if tKnownDye.strName and tKnownDye.strName:len() > 0 then
					sDyeName = tKnownDye.strName
				else
					sDyeName = String_GetWeaselString(Apollo.GetString("CRB_CurlyBrackets"), "", tKnownDye.nRampIndex)
				end
				if sDyeName == tCollection["Dye"..j] then
					bFound = true
					break
				end
			end
			if bFound == false then
				wndNewDyeItem:FindChild("Dye" .. j):FindChild("Icon"):SetSprite("IconSprites:Icon_Windows_UI_CRB_Tooltip_Restricted")
			end
		end
		wndNewDyeItem:FindChild("ItemTextSource"):SetText(tCollection.Source)
	
	end
	wndMainScroll:ArrangeChildrenTiles()
end

function DyeTracker:OnMainNavDyes( wndHandler, wndControl, eMouseButton )
	self:FillDyes2()	
end

function DyeTracker:OnMainNavCols( wndHandler, wndControl, eMouseButton )
	self:FillCollections()
end


function DyeTracker:AlphaBtn_Check( wndHandler, wndControl, eMouseButton )
	table.sort(tMasterDyesDev, function (a,b) return a.sName < b.sName end)
	if self.wndMain:FindChild("MainNavDyes"):IsChecked() then
		self:FillDyes2()
	end
end

function DyeTracker:IndexBtn_Check( wndHandler, wndControl, eMouseButton )
	table.sort(tMasterDyesDev, function (a,b) return a.nIndex < b.nIndex end)
	if self.wndMain:FindChild("MainNavDyes"):IsChecked() then
		self:FillDyes2()
	end
end

function DyeTracker:SortQuality_Checked( wndHandler, wndControl, eMouseButton )
	table.sort(tMasterDyesDev, function (a,b) return a.nQuality < b.nQuality end)
	if self.wndMain:FindChild("MainNavDyes"):IsChecked() then
		self:FillDyes2()
	end
end


function DyeTracker:OnOptionsMenuToggle( wndHandler, wndControl, eMouseButton )
	self.wndMain:FindChild("OptionsContainer"):Show(self.wndMain:FindChild("OptionsBtn"):IsChecked())
end

function DyeTracker:OnFilterChange( wndHandler, wndControl, eMouseButton )
	self:SetVisibleFlag()
	self:FillDyes2()
end

function DyeTracker:SetVisibleFlag()
	local bShowKnown = self.wndMain:FindChild("ToggleKnownBtn"):IsChecked()
	local bShowUnknown = self.wndMain:FindChild("ToggleUnknownBtn"):IsChecked()	

	
	local bShowBox = self.wndMain:FindChild("ToggleBoxBtn"):IsChecked()
	local bShowCollection = self.wndMain:FindChild("ToggleCollectionBtn"):IsChecked()
	local bShowVendor = self.wndMain:FindChild("ToggleVendorBtn"):IsChecked()
	local bShowMisc = self.wndMain:FindChild("ToggleMiscBtn"):IsChecked()
	local bShowMissing = self.wndMain:FindChild("ToggleMissingBtn"):IsChecked()
	local bShowAverage = self.wndMain:FindChild("ToggleAverageBtn"):IsChecked()
	local bShowGood = self.wndMain:FindChild("ToggleGoodBtn"):IsChecked()
	local bShowExcellent = self.wndMain:FindChild("ToggleExcellentBtn"):IsChecked()
	local bShowSuperb = self.wndMain:FindChild("ToggleSuperbBtn"):IsChecked()
	
	
	for i, tMasterDye in ipairs(tMasterDyesDev) do
		local bShowStatus = false
		local bShowSource = false
		local bShowQuality = false
		
		if tMasterDye.Known and bShowKnown then bShowStatus = true end
		if tMasterDye.Known == false and bShowUnknown then bShowStatus = true end


		for j, tSource in ipairs(tMasterDye.Sources) do
			if tSource.nType == 4 and bShowBox then bShowSource = true end
			if tSource.nType == 2 and bShowCollection then bShowSource = true end
			if tSource.nType == 3 and bShowVendor then bShowSource = true end
			if tSource.nType == 5 and bShowMisc then bShowSource = true end
		end		
		if table.getn(tMasterDye.Sources) == 0 and bShowMissing then bShowSource = true end
		
		if tMasterDye.nQuality == 2 and bShowAverage then bShowQuality = true end
		if tMasterDye.nQuality == 3 and bShowGood then bShowQuality = true end
		if tMasterDye.nQuality == 4 and bShowExcellent then bShowQuality = true end
		if tMasterDye.nQuality == 5 and bShowSuperb then bShowQuality = true end
				
		if bShowStatus and bShowSource and bShowQuality then
			tMasterDye.Visible = true
		else
			tMasterDye.Visible = false
		end
	end
end


function DyeTracker:OnOptionsCloseClick( wndHandler, wndControl, eMouseButton )
	self.wndMain:FindChild("OptionsBtn"):SetCheck(false)
	self:OnOptionsMenuToggle()
end

-----------------------------------------------------------------------------------------------
-- DyeTracker Instance
-----------------------------------------------------------------------------------------------
local DyeTrackerInst = DyeTracker:new()
DyeTrackerInst:Init()
